==> plantron-1.1.1.tar.gz: plantron/agentes-aluXXXX.lisp <==
;; Programa: Package con agentes jugadores simples de PlanTron
;;
;; Asignatura: Introducción a la Inteligencia Artificial
;;             ETSII - ULL
;; Práctica: 2 (Lisp I)
;; Autor:    aluXXXX ([Indica aquí tu nombre completo])
;; Fecha:    03/05/2007
;; Comentarios:
;;
;;           Este código supone un esqueleto de ejemplo de el
;;           fichero que ha de contener los agentes jugadores
;;           de PlanTron que el usuario defina. Inicialmente dispone
;;           de un único agente (igual al AGENTE-MITAD que contiene
;;           fichero "pt-agentes-simples.lisp")
;;
;;           Para personalizar este fichero ha de renombrarse,
;;           sustituyendo "aluXXXX" por el lo que desee. Además
;;           en este código ha de reemplazarse dondequiera que
;;           apareza "aluXXXX" o "ALUXXXX" por lo mismo, ya que
;;           el nombre del fichero sin extensión y el nombre
;;           del package aquí incluido han de coincidir (no
;;           sensitivo a mayúsculas).
;;
;;           Para añadir más agentes habrán de generarse las
;;           funciones correspondientes para los agentes, exportarlas
;;           dentro de la sección ":export" de defpackage, e 
;;           incluirlas como sublistas en el contenido de la
;;           constante (DEFCONSTANT) agentes. Para más información
;;           consultar ejemplos de "pt-agentes-simples.lisp" y 
;;           la documentación (DESCRIBE) de las constantes, variables
;;           Y funciones allí utilizadas.
;;
;;           Los agentes definidos utilizan el entorno de juego
;;           PlanTron, definido principalmente por los packages
;;           PT, PT-BASE y ENVIO.
;;
;;           Las funciones accesibles más importantes son:
;;
;;             ALUXXXX-MITAD
;;
;;           Ver la documentación existente para cada una de ellas.
;;
;;           Para su utilización es aconsejable usar el cargador
;;           del entorno de juego PlanTron: "pt-loader.lisp".
;;
;;
;; Dependencias:
;;
;;           Hace uso del paquete que contiene la interface (ENVIO)
;;           con el servidor del juego (PT-BASE).
;;
;;
;; Modificaciones:
;;
;;           - Ver 0.1 (02/05/2007): Primera versión. Extraida
;;             de "pt-agentes-simples.lisp".
;;

;; Por hacer:
;;
;;
;;


;;
;; Definición del package
;; ----------------------

(defpackage "AGENTES-ALUXXXX"
  (:use "COMMON-LISP")
  (:export "AGENTES" "VERSION" "ALUXXXX-MITAD"))


(in-package "AGENTES-ALUXXXX")


;;
;; Constante con lista de agentes presentes en este package
;; --------------------------------------------------------

(defconstant agentes
  '((:turno aluXXXX-mitad))
  "Agentes simples para jugar a PlanTron de aluXXXX.")


;;
;; Constante versión
;; -----------------

(defconstant version "0.1.1"
  "Versión del package AGENTES-ALUXXXX.")


;;
;; Funciones de tratamiento de direcciones y posiciones
;; ----------------------------------------------------

(defun libre-o-planta (tipo)
  "Devuelve T si TIPO (un valor devuelto por ENVIO:CONSULTA-TIPO) se corresponde a una posición vacía (NIL) o a una planta (ENVIO:PLANTA)"
  (or (not tipo) (eq tipo 'envio:planta)))

(defun dir-posible-p (pos dir)
  "Mira si es posible colocarse (sin obstáculo o rastro de agente con que colisionar) en la posición resultante de moverse en la dirección DIR a la posición POS. Devuelve DIR en tal caso y NIL en caso contrario. POS es una posición del tablero dada por una lista con valor de coordenada X y valor de coordenada Y. DIR ha de ser un número entero (dirección) entre 0 y 3 (norte, este, sur y oeste respectivamente)."
  (if (libre-o-planta (envio:consulta-tipo (envio:consulta-pos-tab pos dir)))
      dir))

(defun number-o-0 (elm)
  "Si ELM es un número lo devuelve, sino devuelve 0."
  (if (numberp elm)
      elm
    0))

(defun dir-inc (dir dir-ini)
  "Calcula la dirección DIR relativa a DIR-INI, esto es, gira DIR-INI pasos en sentido agujas del reloj sobre DIR. DIR y DIR-INI han de ser números enteros (direcciones,  numero entero entre 0 y 3: norte, este, sur y oeste respectivamente), si no se consideran cero (norte)."
  (rem (+ (number-o-0 dir) (number-o-0 dir-ini)) 4))

(defun dir-posible-pref (pos lis-dir-pref &optional dir-ini)
  "Busca la primera dirección posible (ver función DIR-POSIBLE-P) alrededor de POS siguiendo el orden de LIS-DIR-PREF, si DIR-INI está instanciado las direcciones de LIS-DIR-PREF son relativas a DIR-INI. POS es una posición del tablero dada por una lista con valor de coordenada X y valor de coordenada Y. LIS-DIR-PREF ha de ser una lista con números enteros (direcciones). DIR-INI ha de ser un número entero (dirección)."
  (if (consp lis-dir-pref)
      (or (dir-posible-p pos (dir-inc (first lis-dir-pref) dir-ini))
	  (dir-posible-pref pos (rest lis-dir-pref) dir-ini))))


;;
;; Agente NSEO
;; -----------

(defun aluXXXX-mitad (id)
  "Agente jugador PlanTron del aluXXXX que sólo busca la mitad."
  (dir-posible-pref (envio:consulta-pos-jug id) '(0 2 1 3)))

==> plantron-1.2.tar.gz: plantron/agentes-aluXXXX.lisp <==
;; Programa: Package con agentes jugadores simples de PlanTron
;;
;; Asignatura: Introducción a la Inteligencia Artificial
;;             ETSII - ULL
;; Práctica: 1 (Lisp I)
;; Autor:    aluXXXX ([Indica aquí tu nombre completo])
;; Fecha:    29/04/2008
;; Comentarios:
;;
;;           Este código supone un esqueleto de ejemplo de el
;;           fichero que ha de contener los agentes jugadores
;;           de PlanTron que el usuario defina. Inicialmente dispone
;;           de un único agente (igual al AGENTE-MITAD que contiene
;;           fichero "pt-agentes-simples.lisp")
;;
;;           Para personalizar este fichero ha de renombrarse,
;;           sustituyendo "aluXXXX" por el lo que desee. Además
;;           en este código ha de reemplazarse dondequiera que
;;           apareza "aluXXXX" o "ALUXXXX" por lo mismo, ya que
;;           el nombre del fichero sin extensión y el nombre
;;           del package aquí incluido han de coincidir (no
;;           sensitivo a mayúsculas).
;;
;;           Para añadir más agentes habrán de generarse las
;;           funciones correspondientes para los agentes, exportarlas
;;           dentro de la sección ":export" de defpackage, e 
;;           incluirlas como sublistas en el contenido de la
;;           constante (DEFCONSTANT) agentes. Para más información
;;           consultar ejemplos de "pt-agentes-simples.lisp" y 
;;           la documentación (DESCRIBE) de las constantes, variables
;;           Y funciones allí utilizadas.
;;
;;           Los agentes definidos utilizan el entorno de juego
;;           PlanTron, definido principalmente por los packages
;;           PT, PT-BASE y ENVIO.
;;
;;           Las funciones accesibles más importantes son:
;;
;;             ALUXXXX-MITAD
;;
;;           Ver la documentación existente para cada una de ellas.
;;
;;           Para su utilización es aconsejable usar el cargador
;;           del entorno de juego PlanTron: "pt-loader.lisp".
;;
;;
;; Dependencias:
;;
;;           Hace uso del paquete que contiene la interface (ENVIO)
;;           con el servidor del juego (PT-BASE).
;;
;;
;; Modificaciones:
;;
;;           - Ver 0.1 (29/04/2008): Primera versión. Extraida
;;             de "pt-agentes-simples.lisp".
;;

;; Por hacer:
;;
;;
;;


;;
;; Definición del package
;; ----------------------

(defpackage "AGENTES-ALUXXXX"
  (:use "COMMON-LISP")
  (:export "AGENTES" "VERSION" "ALUXXXX-MITAD"))


(in-package "AGENTES-ALUXXXX")


;;
;; Constante con lista de agentes presentes en este package
;; --------------------------------------------------------

(defconstant agentes
  '((:turno aluXXXX-mitad))
  "Agentes simples para jugar a PlanTron de aluXXXX.")


;;
;; Constante versión
;; -----------------

(defconstant version "0.1"
  "Versión del package AGENTES-ALUXXXX.")


;;
;; Funciones de tratamiento de direcciones y posiciones
;; ----------------------------------------------------

(defun dir-planta-p (pos dir)
  "Mira si han una planta en la posición resultante de moverse en la dirección DIR a la posición POS. Devuelve DIR en tal caso y NIL en caso contrario. POS es una posición del tablero dada por una lista con valor de coordenada X y valor de coordenada Y. DIR ha de ser un número entero (dirección) entre 0 y 3 (norte, este, sur y oeste respectivamente)."
  (if (eq (envio:consulta-tipo (envio:consulta-pos-tab pos dir)) 'envio:planta)
      dir))

(defun dir-posible-p (pos dir)
  "Mira si es posible colocarse (sin obstáculo o rastro de agente con que colisionar) en la posición resultante de moverse en la dirección DIR a la posición POS. Devuelve DIR en tal caso y NIL en caso contrario. POS es una posición del tablero dada por una lista con valor de coordenada X y valor de coordenada Y. DIR ha de ser un número entero (dirección) entre 0 y 3 (norte, este, sur y oeste respectivamente)."
  (if (not (envio:consulta-tipo (envio:consulta-pos-tab pos dir)))
      dir))

(defun number-o-0 (elm)
  "Si ELM es un número lo devuelve, sino devuelve 0."
  (if (numberp elm)
      elm
    0))

(defun dir-inc (dir dir-ini)
  "Calcula la dirección DIR relativa a DIR-INI, esto es, gira DIR-INI pasos en sentido agujas del reloj sobre DIR. DIR y DIR-INI han de ser números enteros (direcciones,  numero entero entre 0 y 3: norte, este, sur y oeste respectivamente), si no se consideran cero (norte)."
  (rem (+ (number-o-0 dir) (number-o-0 dir-ini)) 4))

(defun dir-posible-pref (pos lis-dir-pref &optional dir-ini)
  "Busca la primera dirección posible (ver función DIR-POSIBLE-P) alrededor de POS siguiendo el orden de LIS-DIR-PREF, si DIR-INI está instanciado las direcciones de LIS-DIR-PREF son relativas a DIR-INI. POS es una posición del tablero dada por una lista con valor de coordenada X y valor de coordenada Y. LIS-DIR-PREF ha de ser una lista con números enteros (direcciones). DIR-INI ha de ser un número entero (dirección)."
  (if (consp lis-dir-pref)
      (or (dir-posible-p pos (dir-inc (first lis-dir-pref) dir-ini))
	  (dir-posible-pref pos (rest lis-dir-pref) dir-ini))))


;;
;; Agente NSEO
;; -----------

(defun aluXXXX-mitad (id)
  "Agente jugador PlanTron del aluXXXX que sólo busca la mitad."
  (dir-posible-pref (envio:consulta-pos-jug id) '(0 2 1 3)))
